#include "node.h"
#include <algorithm>
#include <iostream>
using namespace std;

Node::Node(const Action& action){
	_action = action;
	_preconditions = _action.preconditions();
	_executed = false;
}
Node::Node(){
	_executed = false;
}

Node::Node(const Node& node){
	_action = node._action;
	_preconditions = node._preconditions;
	_executed = node._executed;
}

const Action& Node::action() const{
	return _action;
}


void Node::addChild(Node* node){
	_children.push_back(node);
}

bool Node::executed() const{
	return _executed;
}

const Literals& Node::effects() const{
	return _effects;
}

void Node::execute(const Literals& effects){
	cerr << "Executing: " << _action.name() << endl;
	for (Literals::const_iterator effect = effects.begin(); effect != effects.end(); ++effect){
		cerr << "Satisfied effect: " << *effect << endl;
		_preconditions.erase(_preconditions.find(*effect));
	}
	cerr << "Number of preconditions left: " << _preconditions.size() << endl;
	if(_executed)
		cerr << "Already executed" << endl;
	if ((_preconditions.size() != 0) || _executed)
		return;

	_executed = true;
	int value = _action.execute();
	_effects = _action.effects(value);

	cerr << "Got returnvalue: " << value << ", number of effects: " << _effects.size() << endl;

	for(vector<Node*>::iterator child = _children.begin(); child != _children.end(); ++child){
		(*child)->execute(_effects);
	}
}

StartNode::StartNode(const Literals& init){
	EffectsMap initial;
	initial[0] = init;
	_action = Action("start",Preconditions(),"", initial);
}

EndNode::EndNode(const Literals& goal){
	Preconditions goalState;
	for(Literals::const_iterator g = goal.begin(); g != goal.end(); ++g)
		goalState[*g] = true;
	_action = Action("finish",goalState,"",EffectsMap());
	_preconditions = _action.preconditions();
}

